/*
   rsm.c
   Resource Standard Metrics
   (C) 1996 Robert Mudge
   All Rights Reserved
   M Squared Technologies
   (407) 880-2627
   Email: m2tech@gslink.net
   Use may use this code and program as long as you
   don't use it for commercial use, resell it, publish it
   or change it.  Please contact me for commercial rights to
   use this source.  The use of this program for business
   purposes requires a $25.00 per node license or $500 per
   site license.

   compile with djgpp for DOS or any POSIX unix C compiler

   djgpp or gcc compile script

   call c:\djgpp\setdjgpp c:\djgpp c:/djgpp
   gcc -o rsm -O3 -Wall rsm.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <time.h>

#ifndef MAXPATHLEN
#define MAXPATHLEN 255
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define ON 1
#define OFF 0

#define LINE "------------------------------------"
#define VERSION "1.5"
/*
   version 1.0 determined line counts
   version 1.1 added a new parser and a few keywords
   version 1.2 added the check mode, added the current
               format with key words.
   version 1.3 added goto keyword
               added exit function
               changed output format
   version 1.4 correct statement count to compensate
               for the 2 semicolons in a for loops.
   version 1.5 add metrics for _exit() and abort()
*/

/* Global declarations */
/* unsigned longs for 16 bit compilation */
char filespec [MAXPATHLEN];
unsigned long line_count        = 0;
unsigned long char_count        = 0;
unsigned long commentline_count = 0;
unsigned long semi_count        = 0;
unsigned long codeline_count    = 0;
unsigned long blankline_count   = 0;
unsigned long define_count      = 0;
unsigned long include_count     = 0;
unsigned long string_count      = 0;
unsigned long openparen_count   = 0;
unsigned long closeparen_count  = 0;
unsigned long openbrace_count   = 0;
unsigned long closebrace_count  = 0;
unsigned long openbrack_count   = 0;
unsigned long closebrack_count  = 0;
unsigned long tdirect_count     = 0;
unsigned long default_count     = 0;
unsigned long break_count       = 0;
unsigned long return_count      = 0;
unsigned long const_count       = 0;
unsigned long _exit_count       = 0;
unsigned long abort_count       = 0;
unsigned long exit_count        = 0;
unsigned long while_count       = 0;
unsigned long for_count         = 0;
unsigned long switch_count      = 0;
unsigned long if_count          = 0;
unsigned long else_count        = 0;
unsigned long goto_count        = 0;
unsigned long case_count        = 0;
unsigned long class_count       = 0;
unsigned long struct_count      = 0;
unsigned long template_count    = 0;
unsigned long space_count       = 0;
unsigned long parenline         = 0;
unsigned long braceline         = 0;

unsigned long Tline_count        = 0;
unsigned long Tchar_count        = 0;
unsigned long Tcommentline_count = 0;
unsigned long Tsemi_count        = 0;
unsigned long Tcodeline_count    = 0;
unsigned long Tblankline_count   = 0;
unsigned long Tdefine_count      = 0;
unsigned long Tinclude_count     = 0;
unsigned long Tstring_count      = 0;
unsigned long Topenparen_count   = 0;
unsigned long Tcloseparen_count  = 0;
unsigned long Topenbrace_count   = 0;
unsigned long Tclosebrace_count  = 0;
unsigned long Topenbrack_count   = 0;
unsigned long Tclosebrack_count  = 0;
unsigned long Ttdirect_count     = 0;
unsigned long Tdefault_count     = 0;
unsigned long Tbreak_count       = 0;
unsigned long Treturn_count      = 0;
unsigned long Tconst_count       = 0;
unsigned long T_exit_count        = 0;
unsigned long Tabort_count        = 0;
unsigned long Texit_count        = 0;
unsigned long Twhile_count       = 0;
unsigned long Tfor_count         = 0;
unsigned long Tswitch_count      = 0;
unsigned long Tif_count          = 0;
unsigned long Telse_count        = 0;
unsigned long Tgoto_count        = 0;
unsigned long Tcase_count        = 0;
unsigned long Tclass_count       = 0;
unsigned long Tstruct_count      = 0;
unsigned long Ttemplate_count    = 0;
unsigned long Tspace_count       = 0;
unsigned long Tparenline         = 0;
unsigned long Tbraceline         = 0;

/* system flags */
int verbose_flag     = OFF;
int summary_flag     = ON;
int apos_flag        = OFF;
int cpp_flag         = OFF;
int comment_flag     = OFF;
int string_flag      = OFF;
int code_flag        = OFF;
int param_flag       = OFF;
int check_flag       = OFF;
int comment_begin    = 0;

void
ResetState ( void )
{
  line_count        = 0;
  char_count        = 0;
  commentline_count = 0;
  semi_count        = 0;
  codeline_count    = 0;
  blankline_count   = 0;
  define_count      = 0;
  include_count     = 0;
  string_count      = 0;
  openparen_count   = 0;
  closeparen_count  = 0;
  openbrace_count   = 0;
  closebrace_count  = 0;
  openbrack_count   = 0;
  closebrack_count  = 0;
  tdirect_count     = 0;
  default_count     = 0;
  break_count       = 0;
  return_count      = 0;
  const_count       = 0;
  _exit_count       = 0;
  abort_count       = 0;
  exit_count        = 0;
  while_count       = 0;
  for_count         = 0;
  switch_count      = 0;
  if_count          = 0;
  else_count        = 0;
  goto_count        = 0;
  case_count        = 0;
  class_count       = 0;
  struct_count      = 0;
  template_count    = 0;
  space_count       = 0;
  parenline         = 0;
  braceline         = 0;
  apos_flag         = OFF;
  cpp_flag          = OFF;
  comment_flag      = OFF;
  string_flag       = OFF;
  code_flag         = OFF;
}

void
SetTotal ( void )
{
  line_count        = Tline_count;
  char_count        = Tchar_count;
  commentline_count = Tcommentline_count;
  semi_count        = Tsemi_count;
  codeline_count    = Tcodeline_count;
  blankline_count   = Tblankline_count;
  define_count      = Tdefine_count;
  include_count     = Tinclude_count;
  string_count      = Tstring_count;
  openparen_count   = Topenparen_count;
  closeparen_count  = Tcloseparen_count;
  openbrace_count   = Topenbrace_count;
  closebrace_count  = Tclosebrace_count;
  openbrack_count   = Topenbrack_count;
  closebrack_count  = Tclosebrack_count;
  tdirect_count     = Ttdirect_count;
  _exit_count        = T_exit_count;
  abort_count        = Tabort_count;
  exit_count        = Texit_count;
  default_count     = Tdefault_count;
  break_count       = Tbreak_count;
  return_count      = Treturn_count;
  const_count       = Tconst_count;
  while_count       = Twhile_count;
  for_count         = Tfor_count;
  switch_count      = Tswitch_count;
  if_count          = Tif_count;
  else_count        = Telse_count;
  goto_count        = Tgoto_count;
  case_count        = Tcase_count;
  class_count       = Tclass_count;
  struct_count      = Tstruct_count;
  template_count    = Ttemplate_count;
  space_count       = Tspace_count;
  parenline         = Tparenline;
  braceline         = Tbraceline;
}

void
ShowHeader ( void )
{
  time_t t;
  t = time(NULL);
#ifdef RSMLICENSE
  printf ( "  Resource Standard Metrics For C & C++\n" );
#else
  printf ( "  Resource Standard Metrics For C & C++        Free For Non-commercial Use\n" );
#endif
  printf ( "  Version %s  (C) 1996 Robert Mudge              %s",
           VERSION, ctime(&t) );
  printf ( "  %s%s\n", LINE, LINE );
}

void
PlaceString ( char * string, int length )
{
  int i = 0;
  int sl = 0;
  char str[50];
  strcpy ( str, string );
  sl = strlen ( str );
  if ( strlen(str) < length )
  {
    for ( i=0; i < (length-sl); i++ )
    {
      strcat( str, " ");
    }
  }
  printf ( "%s", str );
}

void
ShowSummary ( char * file )
{
  char buffer[MAXPATHLEN];
  printf ( "\n  File: %s\n", file );
  sprintf( buffer, "%s%ld", "  LoC: ", codeline_count );
  PlaceString ( buffer, 14 );
  sprintf( buffer, "%s%ld", "eLoC: ", (codeline_count-braceline-parenline) );
  PlaceString ( buffer, 14 );
  sprintf( buffer, "%s%ld", "Comment: ", commentline_count );
  PlaceString ( buffer, 18 );
  sprintf( buffer, "%s%ld", "Blank: ", blankline_count );
  PlaceString ( buffer, 15 );
  sprintf( buffer, "%s%ld", "Total: ", line_count );
  PlaceString ( buffer, 15 );
  printf ( "\n" );
}

void
AddTotal( void )
{
  Tline_count        += line_count;
  Tchar_count        += char_count;
  Tcommentline_count += commentline_count;
  Tsemi_count        += semi_count;
  Tcodeline_count    += codeline_count;
  Tblankline_count   += blankline_count;
  Tdefine_count      += define_count;
  Tinclude_count     += include_count;
  Tstring_count      += string_count;
  Topenparen_count   += openparen_count;
  Tcloseparen_count  += closeparen_count;
  Topenbrace_count   += openbrace_count;
  Tclosebrace_count  += closebrace_count;
  Topenbrack_count   += openbrack_count;
  Tclosebrack_count  += closebrack_count;
  Ttdirect_count     += tdirect_count;
  T_exit_count       += _exit_count;
  Tabort_count       += abort_count;
  Texit_count        += exit_count;
  Tdefault_count     += default_count;
  Tbreak_count       += break_count;
  Treturn_count      += return_count;
  Tconst_count       += const_count;
  Twhile_count       += while_count;
  Tfor_count         += for_count;
  Tswitch_count      += switch_count;
  Tif_count          += if_count;
  Telse_count        += else_count;
  Tgoto_count        += goto_count;
  Tcase_count        += case_count;
  Tclass_count       += class_count;
  Tstruct_count      += struct_count;
  Ttemplate_count    += template_count;
  Tspace_count       += space_count;
  Tparenline         += parenline;
  Tbraceline         += braceline;
}

void
ShowVerbose ( char * file )
{
  int l = 38;
  long int eloc = codeline_count-braceline-parenline;
  long int lloc = codeline_count+blankline_count+commentline_count;
  char buff[50];
  memset ( buff, 0, sizeof( buff) );

/* line 1 */
  printf ( "  File: %s\n", file );

/* line 2 */
  PlaceString( "  -------------- LoC ---------------", l );
  printf ( "case .................: %ld\n",  case_count );

/* line 3 */
  sprintf ( buff, "  %s%ld","Lines of Code (LoC)...: ",  codeline_count );
  PlaceString( buff, l );
  printf ( "break ................: %ld\n",  break_count );

/* line 4 */
  sprintf ( buff, "  %s%ld","Lines of just { or } .: ",  braceline );
  PlaceString ( buff, l );
  printf ( "if ...................: %ld\n",  if_count );

/* line 5 */
  sprintf ( buff, "  %s%ld","Lines of just ( or ) .: ",  parenline );
  PlaceString ( buff, l );
  printf ( "else .................: %ld\n",  else_count );

/* line 6 */
  sprintf ( buff, "  %s%ld", "Effective LoC ........: ",  eloc );
  PlaceString ( buff, l );
  printf ( "goto .................: %ld\n",  goto_count );

/* line 7 */
  PlaceString ( "  -------------- Lines -------------", l );
  printf ( "return ...............: %ld\n", return_count );

/* line 8 */
  sprintf ( buff,"  %s%ld","Blank Lines ..........: ",  blankline_count );
  PlaceString ( buff, l );
  printf ( "exit() _exit() abort(): %ld, %ld, %ld\n",
  exit_count, _exit_count, abort_count  );

/* line 9 */
  sprintf ( buff,"  %s%ld","Comment Lines ........: ",  commentline_count );
  PlaceString ( buff, l );
  printf ( "struct ...............: %ld\n",  struct_count );

/* line 10 */
  sprintf ( buff,"  %s%ld","Total Logical Lines ..: ", lloc );
  PlaceString ( buff, l );
  printf ( "class ...........(C++): %ld\n",  class_count );

/* line 11 */
  sprintf ( buff, "  %s%ld","Total Physical Lines .: ", line_count );
  PlaceString ( buff, l );
  printf ( "template ........(C++): %ld\n",  template_count );

/* line 12 */
  PlaceString ( "  ----------- Key Words ------------  ", l );
  printf ( "-------------- Analysis ------------\n" );

/* line 13 */
  sprintf ( buff,"  %s%ld","; ........(Statements): ", 
  semi_count - ( for_count * 2) );
  PlaceString ( buff, l );
  printf ( "Total # directives ...: %ld\n", tdirect_count );
/*  printf ( "String Count .........: %ld\n", string_count ); */

/* line 14 */
  sprintf ( buff, "  %s%ld","#include .............: ", include_count );
  PlaceString ( buff, l );
  printf ( "Paren Count (,) ......: %ld, %ld\n",
           openparen_count, closeparen_count);
/*  printf ( "Space Count ..........: %ld\n", (long)space_count ); */

/* line 15 */
  sprintf ( buff,"  %s%ld","#define ..............: ",  define_count );
  PlaceString ( buff, l );
  printf ( "Brace Count {,} ......: %ld, %ld\n",
           openbrace_count, closebrace_count);
/*  printf ( "Character Count ......: %ld\n", (long)(char_count-space_count) ); */

/* line 16 */
  sprintf ( buff, "  %s%ld", "const ................: ", const_count );
  PlaceString ( buff, l );
  printf ( "Bracket Count [,] ....: %ld, %ld\n",
           openbrack_count, closebrack_count);
/*  printf ( "Total Character Count : %ld\n", (long)char_count ); */

/* line 17 */
  sprintf ( buff, "  %s%ld", "while ................: ",  while_count );
  PlaceString ( buff, l );
  printf ( "Chars / Non-Blank Line: %ld\n",
           (char_count-space_count)/(codeline_count+commentline_count) );

/* line 18 */
  sprintf ( buff, "  %s%ld","for ..................: ", for_count );
  PlaceString ( buff, l );
  printf ( "Code, eCode Lines ....: %4.1f%%, %4.1f%%\n",
           ((float)codeline_count/(float)lloc * 100 ),
           ((float)eloc/(float)lloc * 100 ));

/* line 19 */
  sprintf ( buff,"  %s%ld","switch ...............: ",  switch_count );
  PlaceString ( buff, l );
  printf ( "Comment, Blank Lines .: %4.1f%%, %4.1f%%\n",
           ((float)commentline_count/(float)lloc * 100 ),
           ((float)blankline_count/(float)lloc * 100 ));

/* line 20 */
  sprintf ( buff,"  %s%ld","default ..............: ",  default_count );
  PlaceString ( buff, l );
  printf ( "Characters, Spaces ...: %4.1f%%, %4.1f%%\n",
           ((float)(char_count-space_count)/(float)char_count * 100 ),
           ((float)space_count/(float)char_count * 100 ));

/* line 21 */
  printf ( "  %s%s", LINE, LINE );
}

void
ShowSyntax ( void )
{
  ShowHeader();
  printf ( "  Syntax: rsm <-svhc> filename\n"
           "  -s Summary <default>  -v Verbose"
           "  -c Check/validate  -h Help\n\n" );
}

void
ShowHelp ( void )
{
  ShowSyntax();
  printf (
    "    Resource Standard Metrics provides a standard method to quantify the\n"
    "  quantity and quality of worked performed for a module or a project.\n"
    "  A wild card may be specified for the file name or extension.  The source\n"
    "  is compilable on any ANSI C compiler.  This program will analyze any file\n"
    "  with C or C++ source. The following parameters maybe used:  rsm *.*,\n"
    "  rsm *.c, rsm test.*, rsm -v *.*, rsm -v *.* > output.doc, etc..\n"
    "    Lines of code, 'LoC', can be determined by various techniques.  RSM\n"
    "  will provide raw lines of code, blank lines and comment lines.  Effective\n"
    "  lines of code, eLoC, is defined as raw LoC minus any line which solely\n"
    "  includes '( )' or '{ }'.  Various key words and statements are provided\n"
    "  for code analysis.  Read-ability and code quality can easily be determined\n"
    "  by analyzing comment, white space and key word use within the source code.\n"
    "    The use of this program is FREE for NONCOMMERCIAL purposes.  It may not\n"
    "  be changed or resold.  Please contact the author for commcercial licenses.\n"
    "  Commercial use is $25.00 per node or $500 per site licenses.  Site license\n"
    "  will provide a right to modify the source code as a site desires but the\n"
    "  code may not be resold.  (C) 1996 Robert Mudge, M Squared Technologies\n"
    "         Contact: Robert Mudge, m2tech@gslink.net, (407) 880-2627"
    );
#ifdef __UNIX__
  printf ("\n" );
#endif 
}

void
SetUpParams ( int argc, char * argv[] )
{
  if ( argc == 1 )
  {
    ShowSyntax();
    exit (1);
  }
  if ( argc >= 2 )
  {
    if ( argv[1][0] == '-' )
    {
      param_flag = ON;
      switch ( argv[1][1] )
      {
        case 's':
        case 'S':
        {
          summary_flag = ON;
          verbose_flag = OFF;
        }
        break;

        case 'c':
        case 'C':
        {
          check_flag = ON;
          verbose_flag = ON;
          summary_flag= OFF;
        }
        break;

        case 'V':
        case 'v':
        {
          verbose_flag = ON;
          summary_flag = OFF;
        }
        break;

        case 'H':
        case 'h':
        {
          ShowHelp();
          exit(1);
        }
        break;

        default: /* found something other than these so we have a config file */
        {
          ShowSyntax();
          printf ( "  Parameter -%c Not Supported!\n",argv[1][1]);
          exit(1);
        }
        break;
      }
    }
  }
}

int
SingleChar ( char * line, char token )
{
  int i = 0;
  while ( line[i] == ' ' )
  {
    i++;
  }
  if ( line[i] == token )
  {
    i++;
    while ( line[i] == ' ' )
    {
      i++;
    }
    if ( line[i] == '\0' || line[i] == '\n' )
    {
      return TRUE;
    }
  }
  return FALSE;
}

void
Keyword_Termin ( char * line, int offset, unsigned long * varcount,
                 char char1, char char2 )
{
  int j = offset;
  while ( line[j] == ' ' )
  {
    j++;
  }
  if ( line[j] == char1 || line[j] == char2  )
  {
    *varcount = *varcount + 1;
  }
}


void
Parse_Line ( char * line, int lineno )
{
  char next;
  int  blankline   = TRUE;
  int  codeline    = FALSE;
  int  noteffcode  = FALSE;
  int  commentline = FALSE;
  int  i = 0;
  int  len = strlen ( line ) - 1;

  char_count += len;

  if ( comment_flag == ON )
  {
    commentline = TRUE;
    blankline = FALSE;
  }

  while ( i < len )
  {

    if ( blankline == TRUE )
    {
      if ( line[i] != ' ' )
      {
        blankline = FALSE;
      }
      else if ( comment_flag == ON )
      {
        blankline = FALSE;
      }
    }

    if ( !blankline && !codeline )
    {
      if ( !comment_flag && !cpp_flag )
      {
        if ( line[i] != '/' )
        { 
          if ( line[i+1] != '*' || line[i+1] != '/' )
          {
            codeline = TRUE;
          }
        }
      }
    }

    switch ( line[i] )
    {
      case ' ':
      {
        space_count++;
      }
      break;

      case '"':
      {
        if ( !comment_flag && !cpp_flag && !apos_flag )
        {
          if ( !string_flag )
          {
            string_flag = ON;
          }
          else
          {
            string_flag = OFF;
            string_count++;
          }
        }
        break;
      }

      case '/':
      {
        if ( !string_flag && !comment_flag && !apos_flag && !cpp_flag )
        {
          next = line[i+1];
          if ( next == '*' )
          {
            commentline = TRUE;
            comment_begin = lineno;
            comment_flag = ON;
          }
          else if ( next == '/' )
          {
            cpp_flag = TRUE;
            commentline_count++;
          }
        }
        break;
      }

      case '*':
      {
        if ( !string_flag && comment_flag && !apos_flag && !cpp_flag )
        {
          next = line[i+1];
          if ( next == '/' )
          {
            commentline = TRUE;
            comment_flag = OFF;
            commentline_count = commentline_count + ( lineno - comment_begin ) + 1;
          }
        }
        break;
      }

      case (char)39: /* apostrophe  indicating a character */
      {
        if ( !comment_flag && !string_flag && !cpp_flag )
        {
          if ( apos_flag )
          {
            apos_flag = OFF;
          }
          else
          {
            apos_flag = ON;
          }
        }
        break;
      }

      default:
      {
        if ( !comment_flag && !cpp_flag && !string_flag && !apos_flag )
        {
          switch ( line[i] )
          {
            case '(':
            {
              if ( SingleChar( line, '(' ) == TRUE )
              {
                noteffcode = TRUE;
                parenline++;
              }
              openparen_count++;
              break;
            }
            case ')':
            {
              if ( SingleChar( line, ')' ) == TRUE )
              {
                noteffcode = TRUE;
                parenline++;
              }
              closeparen_count++;
              break;
            }
            case '{':
            {
              if ( SingleChar( line, '{' ) == TRUE )
              {
                noteffcode = TRUE;
                braceline++;
              }
              openbrace_count++;
              break;
            }
            case '}':
            {
              if ( SingleChar( line, '}' ) == TRUE )
              {
                noteffcode = TRUE;
                braceline++;
              }
              closebrace_count++;
              break;
            }
            case '[':
            {
              openbrack_count++;
              break;
            }
            case ']':
            {
              closebrack_count++;
              break;
            }
            case ';':
            {
              semi_count++;
              break;
            }

            case '_':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'e' )
                {
                  if ( line[i+2] == 'x' )
                  {
                    if ( line[i+3] == 'i' )
                    {
                      if ( line[i+4] == 't' )
                      {
                        Keyword_Termin ( line, i+5, &_exit_count,
                                         '(', '\n' );
                      }
                    }
                  }
                }
              }
              break;
            }

            case 'a':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'b' )
                {
                  if ( line[i+2] == 'o' )
                  {
                    if ( line[i+3] == 'r' )
                    {
                      if ( line[i+4] == 't' )
                      {
                        Keyword_Termin ( line, i+5, &abort_count,
                                         '(', '\n' );
                      }
                    }
                  }
                }
              }
              break;
            }

            case 'b':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'r' )
                {
                  if ( line[i+2] == 'e' )
                  {
                    if ( line[i+3] == 'a' )
                    {
                      if ( line[i+4] == 'k' )
                      {
                        if (  line[i+5] == ' ' )
                        {
                            break_count++;
                        }
                        else
                        {
                           Keyword_Termin ( line, i+5, &break_count,
                                        ';', '\n' );
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            case 'c':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'a' )
                {
                  if ( line[i+2] == 's' )
                  {
                    if ( line[i+3] == 'e' )
                    {
                      if ( line[i+4] == ' ' || line[i+4] == '\n' )
                      {
                        case_count++;
                      }
                    }
                  }
                }
                else if ( line[i+1] == 'l' )
                {
                  if ( line[i+2] == 'a' )
                  {
                    if ( line[i+3] == 's' )
                    {
                      if ( line[i+4] == 's' )
                      {
                        if ( line[i+5] == ' ' || line[i+5] == '\n' )
                        {
                          class_count++;
                        }
                      }
                    }
                  }
                }
                else if ( line[i+1] == 'o' )
                {
                  if ( line[i+2] == 'n' )
                  {
                    if ( line[i+3] == 's' )
                    {
                      if ( line[i+4] == 't' )
                      {
                        if ( line[i+5] == ' ' || line[i+5] == '\n' )
                        {
                          const_count++;
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            case 'd':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'e' )
                {
                  if ( line[i+2] == 'f' )
                  {
                    if ( line[i+3] == 'a' )
                    {
                      if ( line[i+4] == 'u' )
                      {
                        if ( line[i+5] == 'l' )
                        {
                          if ( line[i+6] == 't' )
                          {
                             if (  line[i+7] == ' ' )
                             {
                                default_count++;
                             }
                             else
                             {
                               Keyword_Termin ( line, i+7, &default_count,
                                            ':', '\n' );
                             }
                          }
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            case 'e':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'l' )
                {
                  if ( line[i+2] == 's' )
                  {
                    if ( line[i+3] == 'e' )
                    {
                      if ( line[i+4] == ' ' || line[i+4] == '\n' )
                      {
                        else_count++;
                      }
                    }
                  }
                }
                else
                {
                  if ( line[i+1] == 'x' )
                  {
                    if ( line[i+2] == 'i' )
                    {
                      if ( line[i+3] == 't' )
                      {
                        Keyword_Termin ( line, i+4, &exit_count,
                                         '(', '\n' );
                      }
                    }
                  }
                }
              }
              break;
            }
            case 'f':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'o' )
                {
                  if ( line[i+2] == 'r' )
                  {
                    Keyword_Termin ( line, i+3, &for_count,
                                     '(', '\n' );
                  }
                }
              }
              break;
            }
            case 'g':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'o' )
                {
                  if ( line[i+2] == 't' )
                  {
                    if ( line[i+3] == 'o' )
                    {
                      if ( line[i+4] == ' ' || line[i+4] == '\n' )
                      {
                        goto_count++;
                      }
                    }
                  }
                }
              }
              break;
            }
            case 'i':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'f' )
                {
                  Keyword_Termin ( line, i+2, &if_count,
                                   '(', '\n' );
                }
              }
              break;
            }
            case 'r':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'e' )
                {
                  if ( line[i+2] == 't' )
                  {
                    if ( line[i+3] == 'u' )
                    {
                      if ( line[i+4] == 'r' )
                      {
                        if ( line[i+5] == 'n' )
                        {
                           if (  line[i+6] == ' ' )
                           {
                              return_count++;
                           }
                           else
                           {
                             Keyword_Termin ( line, i+6, &return_count,
                                          '(', '\n' );
                           }
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            case 's':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'w' )
                {
                  if ( line[i+2] == 'i' )
                  {
                    if ( line[i+3] == 't' )
                    {
                      if ( line[i+4] == 'c' )
                      {
                        if ( line[i+5] == 'h' )
                        {
                          Keyword_Termin ( line, i+6, &switch_count,
                                           '(', '\n' );
                        }
                      }
                    }
                  }
                }
                else if ( line[i+1] == 't' )
                {
                  if ( line[i+2] == 'r' )
                  {
                    if ( line[i+3] == 'u' )
                    {
                      if ( line[i+4] == 'c' )
                      {
                        if ( line[i+5] == 't' )
                        {
                          if ( line[i+6] == ' ' || line[i+6] == '\n' )
                          {
                            struct_count++;
                          }
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            case 't':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'e' )
                {
                  if ( line[i+2] == 'm' )
                  {
                    if ( line[i+3] == 'p' )
                    {
                      if ( line[i+4] == 'l' )
                      {
                        if ( line[i+5] == 'a' )
                        {
                          if ( line[i+6] == 't' )
                          {
                            if ( line[i+7] == 'e' )
                            {
                              Keyword_Termin ( line, i+8, &template_count,
                                               '<', '\n' );
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            case 'w':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                if ( line[i+1] == 'h' )
                {
                  if ( line[i+2] == 'i' )
                  {
                    if ( line[i+3] == 'l' )
                    {
                      if ( line[i+4] == 'e' )
                      {
                        Keyword_Termin ( line, i+5, &while_count,
                                         '(', '\n' );
                      }
                    }
                  }
                }
              }
              break;
            }
            case '#':
            {
              if ( i == 0 || line[i-1] == ' ' )
              {
                tdirect_count++;
                if ( line[i+1] == 'd' )
                {
                  if ( line[i+2] == 'e' )
                  {
                    if ( line[i+3] == 'f' )
                    {
                      if ( line[i+4] == 'i' )
                      {
                        if ( line[i+5] == 'n' )
                        {
                          if ( line[i+6] == 'e' )
                          {
                            if ( line[i+7] == ' ' || line[i+7] == '\n' )
                            {
                              define_count++;
                            }
                          }
                        }
                      }
                    }
                  }
                }
                else if ( line[i+1] == 'i' )
                {
                  if ( line[i+2] == 'n' )
                  {
                    if ( line[i+3] == 'c' )
                    {
                      if ( line[i+4] == 'l' )
                      {
                        if ( line[i+5] == 'u' )
                        {
                          if ( line[i+6] == 'd' )
                          {
                            if ( line[i+7] == 'e' )
                            {
                              if ( line[i+8] == ' ' || line[i+8] == '\n' )
                              {
                                include_count++;
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
              break;
            }
            default:
            break;
          }
        }
        break;
      }
    }
    i++;
  }

  if ( check_flag == TRUE )
  {
    printf ( "\nLine %ld: %sState:", line_count, line );
    if ( blankline == TRUE )
    {
      printf ( " Blank" );
    }
    if ( codeline == TRUE )
    {
      printf ( " Code" );
      if ( noteffcode == TRUE )
      {
        printf ( " Non-Effective" );
      }
    }
    if ( commentline == TRUE )
    {
      printf ( " C-Comment" );
    }
    if ( cpp_flag== TRUE )
    {
      printf ( " CPP-Comment" );
    }
    printf ( "\n" );
  }
  if ( blankline == TRUE )
  {
    blankline_count++;
  }
  if ( codeline == TRUE )
  {
    codeline_count++;
  }
  cpp_flag = OFF;
}

int
main ( int argc, char ** argv )
{
  FILE * file_ptr = NULL;
  char filename[MAXPATHLEN];
  char line [MAXPATHLEN];
  int param_i = 0;

  SetUpParams ( argc, argv );
  ShowHeader();

  if ( param_flag == ON )
  {
    param_i = 2;
  }
  else
  {
    param_i = 1;
  }

  while ( param_i < argc )
  {
    strcpy ( filename, argv[param_i] );
    if ( filename )
    {
      file_ptr = fopen ( argv[param_i], "r" );
      if ( !file_ptr )
      {
        fprintf ( stderr,
                  "\n  Problem opening input file: %s\n\n", filename );
      }
      else
      {
        while ( TRUE )
        {

          fgets( line, sizeof(line), file_ptr );

          if ( feof ( file_ptr ) )
          {
            break;
          }

          line_count++;

          Parse_Line ( line, line_count );

        }
        if ( summary_flag )
        {
          ShowSummary ( filename );
        }
        else
        {
          ShowVerbose ( filename );
          if ( (param_flag == ON && argc > 3) ||
               (param_flag == OFF && argc > 2) )
          {
            printf( "\n" );
          }
        }
        AddTotal();
        ResetState();
        fclose ( file_ptr );
      }
    }
    param_i++;
  }
  if ( (param_flag == ON && argc > 3) ||
       (param_flag == OFF && argc > 2) )
  {
    SetTotal();
    if ( summary_flag )
    {
      ShowSummary ( "Total For All Files" );
    }
    else
    {
      ShowVerbose ( "Total For All Files" );
    }
  }
#ifdef __UNIX__
  printf ("\n" );
#endif 
  return (1);
}
